// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"
#include "DriverInterfaceCypher.h"
#include "DriverInterfaceCommon.h"

    
// =========================================================================
BOOL driver_CypherLoadDLL(WCHAR* Filename, MODULE_DETAILS_CYPHER* DLLDetails)
{
    BOOL retval = TRUE;

    DEBUGOUTMAINDRV(DEBUGLEV_ENTER, (TEXT("driver_CypherLoadDLL\n")));

    // Get library path and filename...
    if (retval)
        {
        // +1 for terminating NULL
        DLLDetails->Filename = calloc((wcslen(Filename) + 1), sizeof(DLLDetails->Filename[0]));
        if (DLLDetails->Filename == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Unable to malloc memory to copy driver filename\n")));
            retval = FALSE;
            }
        else
            {
            wcscpy(DLLDetails->Filename, Filename);
            }
        }

    // Load library...
    if (retval)
        {
        DLLDetails->Lib = LoadLibrary(DLLDetails->Filename);
        if (DLLDetails->Lib == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to load library\n")));
            retval = FALSE;
            }
        }

    // Get library function address...
    if (retval)
        {
        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_IDENTIFYDRIVER));
        DLLDetails->FnIdentifyDriver = (PCypherDLLFnIdentifyDriver)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_IDENTIFYDRIVER
                                        );
        if (DLLDetails->FnIdentifyDriver == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_IDENTIFYSUPPORTED));
        DLLDetails->FnIdentifySupported = (PCypherDLLFnIdentifySupported)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_IDENTIFYSUPPORTED
                                        );
        if (DLLDetails->FnIdentifySupported == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_GETCYPHERDETAILS));
        DLLDetails->FnGetCypherDetails = (PCypherDLLFnGetCypherDetails)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_GETCYPHERDETAILS
                                        );
        if (DLLDetails->FnGetCypherDetails == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_ENCRYPT));
        DLLDetails->FnEncrypt = (PCypherDLLFnEncrypt)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_ENCRYPT
                                        );
        if (DLLDetails->FnEncrypt == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_ENCRYPTWITHASCII));
        DLLDetails->FnEncryptWithASCII = (PCypherDLLFnEncryptWithASCII)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_ENCRYPTWITHASCII
                                        );
        if (DLLDetails->FnEncryptWithASCII == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_DECRYPT));
        DLLDetails->FnDecrypt = (PCypherDLLFnDecrypt)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_DECRYPT
                                        );
        if (DLLDetails->FnDecrypt == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        DEBUGOUTMAINDRV(DEBUGLEV_INFO, (TEXT("Getting proc address for: %ls\n"), DLLEXPORT_CYPHER_DECRYPTWITHASCII));
        DLLDetails->FnDecryptWithASCII = (PCypherDLLFnDecryptWithASCII)GetProcAddress(
                                         DLLDetails->Lib,
                                         DLLEXPORT_CYPHER_DECRYPTWITHASCII
                                        );
        if (DLLDetails->FnDecryptWithASCII == NULL)
            {
            DEBUGOUTMAINDRV(DEBUGLEV_ERROR, (TEXT("FAILED to get proc address\n")));
            retval = FALSE;
            }

        }


    // In case of failure, shutdown any loaded lib, overwrite, and free off...
    if (!(retval))
        {
        driver_CypherUnloadDLL(DLLDetails);
        }

    DEBUGOUTMAINDRV(DEBUGLEV_EXIT, (TEXT("driver_CypherLoadDLL\n")));
    return retval;
}


// =========================================================================
void driver_CypherUnloadDLL(MODULE_DETAILS_CYPHER* DLLDetails)
{
    DEBUGOUTMAINDRV(DEBUGLEV_ENTER, (TEXT("driver_CypherUnloadDLL\n")));

    if (DLLDetails != NULL)
        {
      	// DLL handle
        if ((DLLDetails->Lib) != NULL)
            {
	        FreeLibrary(DLLDetails->Lib);
            DLLDetails->Lib = NULL;
            }

        SecZeroAndFreeWCHARMemory(DLLDetails->Filename);

        SecZeroMemory(DLLDetails, sizeof(*DLLDetails));
        }

    DEBUGOUTMAINDRV(DEBUGLEV_EXIT, (TEXT("driver_CypherUnloadDLL\n")));
}


// =========================================================================
// malloc a CYPHER_DRIVER_INFO structure, and populate as appropriate; returning
// a pointer to the structure in driverInfo
BOOL driver_CypherGetDriverDetails(WCHAR* CypherFilename, CYPHER_DRIVER_INFO* driverInfo)
{
    BOOL retval = FALSE;
    MODULE_DETAILS_CYPHER DLLDetails;
    DIOC_CYPHER_IDENTIFYDRIVER identifyBuf;
    DWORD supportedBufSize;
    DIOC_CYPHER_IDENTIFYSUPPORTED* supportedBuf;
    CYPHER* cypherArray;

    DEBUGOUTGUI(DEBUGLEV_ENTER, (TEXT("driver_CypherGetDriverDetails\n")));

    if (driver_CypherLoadDLL(CypherFilename, &DLLDetails))
        {
        if (DLLDetails.FnIdentifyDriver(&identifyBuf) != ERROR_SUCCESS)
            {
            DEBUGOUTGUI(DEBUGLEV_ERROR, (TEXT("Call to FnIdentifyDriver FAILED!")));
            }
        else
            {
            supportedBufSize = (
                                sizeof(*supportedBuf) -
                                sizeof(CYPHER) +                                 
                                (sizeof(CYPHER) * identifyBuf.CountCyphers)
                               );
            supportedBuf = malloc(supportedBufSize);
            if (supportedBuf == NULL)
                {
                DEBUGOUTGUI(DEBUGLEV_ERROR, (TEXT("Unable to malloc DIOCBuffer FnIdentifySupported call\n")));
                }
            else
                {
                if (DLLDetails.FnIdentifySupported(
                                                   supportedBufSize, 
                                                   supportedBuf
                                                  ) != ERROR_SUCCESS)
                    {
                    DEBUGOUTGUI(DEBUGLEV_ERROR, (TEXT("Call to FnIdentifySupported FAILED!")));
                    }
                else
                    {
                    cypherArray = calloc(supportedBuf->BufCount, sizeof(CYPHER));
                    if (cypherArray == NULL)
                        {
                        DEBUGOUTGUI(DEBUGLEV_ERROR, (TEXT("Unable to malloc buffer for list of supported to be returned\n")));
                        }
                    else
                        {
                        driverInfo->DriverGUID = identifyBuf.DriverGUID;
                        memcpy(
                               driverInfo->DriverTitle,
                               identifyBuf.Title,
                               sizeof(identifyBuf.Title)
                              );
                        driverInfo->DriverVersionID = identifyBuf.VersionID;
                        driverInfo->CypherCount = identifyBuf.CountCyphers;

                        memcpy(
                               cypherArray, 
                               supportedBuf->Cyphers,
                               (supportedBuf->BufCount * sizeof(*cypherArray))
                              );

                        driverInfo->CypherDetails = cypherArray;

                        retval = TRUE;
                        }
                    }

                SecZeroAndFreeMemory(supportedBuf, supportedBufSize);
                }

            SecZeroMemory(&identifyBuf, sizeof(identifyBuf));
            }

        driver_CypherUnloadDLL(&DLLDetails);
        }

    DEBUGOUTGUI(DEBUGLEV_EXIT, (TEXT("driver_CypherGetDriverDetails\n")));
    return retval;
}


// =========================================================================
void driver_CypherFreeDriverDetails(CYPHER_DRIVER_INFO* driverInfo)
{
    if (driverInfo != NULL)
        {
        SecZeroMemory(driverInfo, sizeof(*driverInfo));
        SecZeroAndFreeMemory(
                             driverInfo->CypherDetails, 
                             (driverInfo->CypherCount * sizeof(CYPHER))
                            );
        }
}


// =========================================================================
// EncryptNotDecrypt - Set to TRUE to encrypt, FALSE to decrypt
BOOL _driver_CypherEncryptDecryptData(
    BOOL EncryptNotDecrypt,
    WCHAR* CypherDriverFilename,
    GUID CypherGUID,
    unsigned int KeySize, // In *bits*
    FREEOTFEBYTE* Key,
    unsigned int IVSize, // In *bits*
    FREEOTFEBYTE* IV,
    unsigned int BufferSize, // In bytes
    FREEOTFEBYTE* BufferIn,
    FREEOTFEBYTE* BufferOut
)
{
    BOOL retval = FALSE;
    MODULE_DETAILS_CYPHER DLLDetails;
    DIOC_CYPHER_DATA_IN* DIOCIn;
    unsigned int DIOCInSize;    
    FREEOTFEBYTE* ptrKey;
    FREEOTFEBYTE* ptrIV;
    FREEOTFEBYTE* ptrData;

    DEBUGOUTGUI(DEBUGLEV_ENTER, (TEXT("_driver_CypherEncryptDecryptData\n")));

    if (driver_CypherLoadDLL(CypherDriverFilename, &DLLDetails))
        {
        DIOCInSize = sizeof(*DIOCIn) - 
                     sizeof(DIOCIn->Key) - 
                     sizeof(DIOCIn->IV) - 
                     sizeof(DIOCIn->Data) +
                     (KeySize / 8) +
                     (IVSize / 8) + 
                     BufferSize;

        DIOCIn = malloc(DIOCInSize);
        if (DIOCIn == NULL) 
            {
            DEBUGOUTGUI(DEBUGLEV_ERROR, (TEXT("Unable to malloc DIOC in buffer\n")));
            }
        else
            {
            DIOCIn->CypherGUID = CypherGUID;
            DIOCIn->KeyLength = KeySize;
            DIOCIn->IVLength = IVSize;
            DIOCIn->DataLength = BufferSize;

            ptrKey  = DIOCIn->Key;
            ptrIV   = ptrKey + (KeySize / 8);
            ptrData = ptrIV + (IVSize / 8);

            memcpy(ptrKey, Key, (KeySize / 8));
            memcpy(ptrIV, IV, (IVSize / 8));
            memcpy(ptrData, BufferIn, BufferSize);
                
            if (EncryptNotDecrypt)
                {
                retval = (DLLDetails.FnEncrypt(
                                               DIOCInSize,
                                               DIOCIn,
                                               BufferSize,
                                               (DIOC_CYPHER_DATA_OUT*)BufferOut
                                              ) == ERROR_SUCCESS);
                }
            else
                {
                retval = (DLLDetails.FnDecrypt(
                                               DIOCInSize,
                                               DIOCIn,
                                               BufferSize,
                                               (DIOC_CYPHER_DATA_OUT*)BufferOut
                                              ) == ERROR_SUCCESS);
                }

            SecZeroAndFreeMemory(DIOCIn, DIOCInSize);
            }

        driver_CypherUnloadDLL(&DLLDetails);
        }

    DEBUGOUTGUI(DEBUGLEV_EXIT, (TEXT("_driver_CypherEncryptDecryptData\n")));
    return retval;
}


// =========================================================================
BOOL driver_CypherEncryptData(
    WCHAR* CypherDriverFilename,
    GUID CypherGUID,
    unsigned int keySize, // In *bits*
    FREEOTFEBYTE* Key,
    unsigned int IVSize, // In *bits*
    FREEOTFEBYTE* IV,
    unsigned int blockSize, // In bytes
    FREEOTFEBYTE* plaintextBuffer,
    FREEOTFEBYTE* encryptedBuffer
)
{
    return _driver_CypherEncryptDecryptData(
                               TRUE,
                               CypherDriverFilename,
                               CypherGUID,
                               keySize, // In *bits*
                               Key,
                               IVSize, // In *bits*
                               IV,
                               blockSize, // In bytes
                               plaintextBuffer,
                               encryptedBuffer
                              );
}


// =========================================================================
BOOL driver_CypherDecryptData(
    WCHAR* CypherDriverFilename,
    GUID CypherGUID,
    unsigned int keySize, // In *bits*
    FREEOTFEBYTE* Key,
    unsigned int IVSize, // In *bits*
    FREEOTFEBYTE* IV,
    unsigned int blockSize, // In bytes
    FREEOTFEBYTE* encryptedBuffer,
    FREEOTFEBYTE* plaintextBuffer
)
{
    return _driver_CypherEncryptDecryptData(
                               FALSE,
                               CypherDriverFilename,
                               CypherGUID,
                               keySize, // In *bits*
                               Key,
                               IVSize, // In *bits*
                               IV,
                               blockSize, // In bytes
                               encryptedBuffer,
                               plaintextBuffer
                              );
}


// =========================================================================
void driver_CypherPrettyprintAlgTitle(
    CYPHER* cypherInfo, 
    WCHAR* buffer,
    int bufferSize  // In bytes
)
{
    WCHAR strMode[10];  // Enough to hold any cypher mode as a string
    WCHAR tmpStrBuf[1024];  // 1024 relativly arbitary; must be big enough to
                            // store a full prettyprinted hash/cypher title

    // Convert straight ASCII title to UNICODE
    memset(tmpStrBuf, 0, sizeof(tmpStrBuf));
    mbstowcs(
             tmpStrBuf, 
             cypherInfo->Title, 
             strlen(cypherInfo->Title)
            ); 

    // Conver cypher mode to UNICODE string representation
    memset(strMode, 0, sizeof(strMode));
    driver_CypherPrettyprintAlgMode(
                                    cypherInfo->Mode,
                                    strMode
                                   );

    // Prettyprint to buffer
    _snwprintf(
              buffer,
              (bufferSize / sizeof(buffer[0])),
              STR_FORMAT_FULL_CYPHER_TITLE,
              tmpStrBuf,
              strMode,
              cypherInfo->KeySize,
              cypherInfo->BlockSize
             );

    SecZeroMemory(tmpStrBuf, sizeof(tmpStrBuf));
}


// =========================================================================
// !! IMPORTANT !!
// Buffer **MUST** be large enough to store mode as UNICODE string!
// (Obvious, but...)
void driver_CypherPrettyprintAlgMode(
    CYPHER_MODE mode,
    WCHAR* buffer
)
{
    switch (mode)
        {
        case CYPHER_MODE_NONE:
            {
            wcscpy(buffer, TEXT("None"));
            break;
            }

        case CYPHER_MODE_ECB:
            {
            wcscpy(buffer, TEXT("ECB"));
            break;
            }

        case CYPHER_MODE_CBC:
            {
            wcscpy(buffer, TEXT("CBC"));
            break;
            }

        default:
            {
            wcscpy(buffer, STR_UNKNOWN);
            break;
            }
        }

}


// =========================================================================
BOOL
driver_CypherGetImplDetails(
    WCHAR* CypherFilename, 
    GUID* CypherImpl, 
    CYPHER* ImplDetails
)
{
    BOOL retval = FALSE;
    CYPHER_DRIVER_INFO cypherInfo;
    unsigned int i;

    DEBUGOUTGUI(DEBUGLEV_ENTER, (TEXT("driver_CypherGetImplDetails\n")));

    if (driver_CypherGetDriverDetails(
                                    CypherFilename,
                                    &cypherInfo
                                   ))
        {
        for (i = 0; i < cypherInfo.CypherCount; i++)
            {
            if (IsEqualGUID(
                            CypherImpl,
                            &(cypherInfo.CypherDetails[i].CypherGUID)
                           ))
                {
                *ImplDetails = cypherInfo.CypherDetails[i];
                retval = TRUE;
                break;
                }
            }

        driver_CypherFreeDriverDetails(&cypherInfo);
        }

    DEBUGOUTGUI(DEBUGLEV_EXIT, (TEXT("driver_CypherGetImplDetails\n")));
    return retval;
}


// =========================================================================
// =========================================================================
